@@ -238,7 +238,14 @@ class DFMessage(object):
                     noisy_nan = "\x7f\xf8\x00\x00\x00\x00\x00\x00"
                 if struct.pack(">d", val) != noisy_nan:
                     val = "qnan"
-            ret += "%s : %s, " % (c, val)
+
+            if is_py3:
+                ret += "%s : %s, " % (c, val)
+            else:
+                try:
+                    ret += "%s : %s, " % (c, val)
+                except UnicodeDecodeError:
+                    ret += "%s : %s, " % (c, to_string(val))
             col_count += 1
         if col_count != 0:
             ret = ret[:-2]
@@ -666,6 +673,10 @@ class DFReader(object):
             self.flightmode = mavutil.mode_string_px4(m.MainState)
         if type == 'PARM' and getattr(m, 'Name', None) is not None:
             self.params[m.Name] = m.Value
+            if hasattr(m,'Default') and not math.isnan(m.Default):
+                if not hasattr(self,'param_defaults'):
+                    self.param_defaults = {}
+                self.param_defaults[m.Name] = m.Default
         self._set_time(m)
 
     def recv_match(self, condition=None, type=None, blocking=False):
