@@ -5,6 +5,12 @@ extern "C" {
 #ifndef _CHECKSUM_H_
 #define _CHECKSUM_H_
 
+// Visual Studio versions before 2010 don't have stdint.h, so we just error out.
+#if (defined _MSC_VER) && (_MSC_VER < 1600)
+#error "The C-MAVLink implementation requires Visual Studio 2010 or greater"
+#endif
+
+#include <stdint.h>
 
 /**
  *
@@ -15,9 +21,8 @@ extern "C" {
 #define X25_INIT_CRC 0xffff
 #define X25_VALIDATE_CRC 0xf0b8
 
-#ifndef HAVE_CRC_ACCUMULATE
 /**
- * @brief Accumulate the X.25 CRC by adding one char at a time.
+ * @brief Accumulate CRC16_MCRF4XX by adding one char at a time.
  *
  * The checksum function adds the hash of one char at a time to the
  * 16 bit checksum (uint16_t).
@@ -34,13 +39,11 @@ static inline void crc_accumulate(uint8_t data, uint16_t *crcAccum)
         tmp ^= (tmp<<4);
         *crcAccum = (*crcAccum>>8) ^ (tmp<<8) ^ (tmp <<3) ^ (tmp>>4);
 }
-#endif
-
 
 /**
- * @brief Initiliaze the buffer for the X.25 CRC
+ * @brief Initialize the buffer for the MCRF4XX CRC16
  *
- * @param crcAccum the 16 bit X.25 CRC
+ * @param crcAccum the 16 bit MCRF4XX CRC16
  */
 static inline void crc_init(uint16_t* crcAccum)
 {
@@ -49,7 +52,7 @@ static inline void crc_init(uint16_t* crcAccum)
 
 
 /**
- * @brief Calculates the X.25 checksum on a byte buffer
+ * @brief Calculates the MCRF4XX CRC16 checksum on a byte buffer
  *
  * @param  pBuffer buffer containing the byte array to hash
  * @param  length  length of the byte array
@@ -65,9 +68,8 @@ static inline uint16_t crc_calculate(const uint8_t* pBuffer, uint16_t length)
         return crcTmp;
 }
 
-
 /**
- * @brief Accumulate the X.25 CRC by adding an array of bytes
+ * @brief Accumulate the MCRF4XX CRC16 by adding an array of bytes
  *
  * The checksum function adds the hash of one char at a time to the
  * 16 bit checksum (uint16_t).
@@ -75,7 +77,7 @@ static inline uint16_t crc_calculate(const uint8_t* pBuffer, uint16_t length)
  * @param data new bytes to hash
  * @param crcAccum the already accumulated checksum
  **/
-static inline void crc_accumulate_buffer(uint16_t *crcAccum, const char *pBuffer, uint16_t length)
+static inline void crc_accumulate_buffer(uint16_t *crcAccum, const char *pBuffer, uint8_t length)
 {
 	const uint8_t *p = (const uint8_t *)pBuffer;
 	while (length--) {
@@ -83,6 +85,9 @@ static inline void crc_accumulate_buffer(uint16_t *crcAccum, const char *pBuffer
         }
 }
 
+
+
+
 #endif /* _CHECKSUM_H_ */
 
 #ifdef __cplusplus
