@@ -1,5 +1,5 @@
 '''
-MAVLink X25 CRC code
+MAVLink CRC-16/MCRF4XX code
 
 Copyright Andrew Tridgell
 Released under GNU LGPL version 3 or later
@@ -8,7 +8,7 @@ from builtins import object
 
 
 class x25crc(object):
-    '''x25 CRC - based on checksum.h from mavlink library'''
+    '''CRC-16/MCRF4XX - based on checksum.h from mavlink library'''
     def __init__(self, buf=None):
         self.crc = 0xffff
         if buf is not None:
