@@ -3,8 +3,11 @@
 '''
 show changes in flight modes
 '''
+from __future__ import print_function
 
-import sys, time, datetime, os
+import datetime
+import os
+import time
 
 from argparse import ArgumentParser
 parser = ArgumentParser(description=__doc__)
@@ -49,7 +52,7 @@ def flight_modes(logfile):
             time_in_mode[previous_mode] = time_in_mode[previous_mode] + (m._timestamp - mode_start_timestamp)
 
             #figure out how many seconds per percentage point so I can
-            #caculate how many seconds for the final mode
+            #calculate how many seconds for the final mode
             if (seconds_per_percent == -1 and previous_percent != -1
                     and previous_percent != mlog.percent):
                 seconds_per_percent = (m._timestamp - mode_start_timestamp) / (mlog.percent - previous_percent)
@@ -60,8 +63,8 @@ def flight_modes(logfile):
         previous_percent = mlog.percent
 
     #put a whitespace line before the per-mode report
-    print
-    print "Time per mode:"
+    print()
+    print("Time per mode:")
 
     #need to get the time in the final mode
     if (seconds_per_percent != -1):
@@ -70,14 +73,14 @@ def flight_modes(logfile):
         time_in_mode[previous_mode] = time_in_mode[previous_mode] + seconds_remaining
 
         total_flight_time = 0
-        for key, value in time_in_mode.iteritems():
+        for key, value in time_in_mode.items():
             total_flight_time = total_flight_time + value
 
-        for key, value in time_in_mode.iteritems():
+        for key, value in time_in_mode.items():
             print('%-12s %s %.2f%%' % (key, str(datetime.timedelta(seconds=int(value))), (value / total_flight_time) * 100.0))
     else:
         #can't print time in mode if only one mode during flight
-        print previous_mode, " 100% of flight time"
+        print(previous_mode, " 100% of flight time")
 
 for filename in args.logs:
     flight_modes(filename)
