@@ -11,6 +11,7 @@ import time
 from argparse import ArgumentParser
 parser = ArgumentParser(description=__doc__)
 parser.add_argument("-c", "--changes", dest="changesOnly", default=False, action="store_true", help="Show only changes to parameters.")
+parser.add_argument("--qgc", action='store_true', help="output in QGC-friendly format")
 parser.add_argument("logs", metavar="LOG", nargs="+")
 
 args = parser.parse_args()
@@ -46,8 +47,21 @@ total = 0.0
 for filename in args.logs:
     mavparms(filename)
 
+if args.qgc:
+    # see https://dev.qgroundcontrol.com/master/en/file_formats/parameters.html
+    print("""
+# # Vehicle-Id Component-Id Name Value Type
+""")
+
 if (args.changesOnly is False):
     keys = list(parms.keys())
     keys.sort()
     for p in keys:
-        print("%-15s %.6f" % (p, parms[p]))
+        if args.qgc:
+            MAV_PARAM_TYPE_REAL32 = 9
+            sysid = 1
+            compid = 1
+            print("%u %u %-15s %.6f %u" %
+                  (sysid, compid, p, parms[p], MAV_PARAM_TYPE_REAL32))
+        else:
+            print("%-15s %.6f" % (p, parms[p]))
