@@ -3,14 +3,19 @@
 '''
 convert a MAVLink tlog file to a MATLab mfile
 '''
+from __future__ import print_function
+from builtins import range
 
-import sys, os
+import os
 import re
+import sys
 from pymavlink import mavutil
 
 def process_tlog(filename):
     '''convert a tlog to a .m file'''
 
+    is_py2 = sys.version_info < (3,0)
+
     print("Processing %s" % filename)
 
     mlog = mavutil.mavlink_connection(filename, dialect=args.dialect, zero_time_base=True)
@@ -29,7 +34,7 @@ def process_tlog(filename):
     basename = '.'.join(tail.split('.')[:-1])
     mfilename = re.sub('[\.\-\+\*]','_', basename) + '.m'
     # Octave also doesn't like files that don't start with a letter
-    if (re.match('^[a-zA-z]', mfilename) == None):
+    if re.match('^[a-zA-z]', mfilename) is None:
         mfilename = 'm_' + mfilename
 
     if head is not None:
@@ -57,9 +62,14 @@ def process_tlog(filename):
 
         if mtype not in type_counters:
             type_counters[mtype] = 0
-            f.write("%s.heading = {'timestamp'" % mtype)
+            f.write("%s.columns = {'timestamp'" % mtype)
             for field in fieldnames:
                 val = getattr(m, field)
+
+                if is_py2:
+                    if isinstance(val,unicode): # NOQA
+                        val = str(val)
+
                 if not isinstance(val, str):
                     if type(val) is not list:
                         f.write(",'%s'" % field)
@@ -72,6 +82,11 @@ def process_tlog(filename):
         f.write("%s.data(%u,:) = [%f" % (mtype, type_counters[mtype], m._timestamp))
         for field in m._fieldnames:
             val = getattr(m, field)
+
+            if is_py2:
+                if isinstance(val,unicode): # NOQA
+                    val = str(val)
+
             if not isinstance(val, str):
                 if type(val) is not list:
                     f.write(",%.20g" % val)
